/*
 * Common clock and power management (CPM) abstractions.
 *
 * Copyright (C) 2017, 2018, 2020, 2021, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/cpm-common.h>
#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

/* A transitional abstraction supporting older CPM abstractions. */

class Cpm_chip_base
{
public:
  virtual int have_clock(enum Clock_identifiers clock) = 0;
  virtual void start_clock(enum Clock_identifiers clock) = 0;
  virtual void stop_clock(enum Clock_identifiers clock) = 0;
};

/* A simple abstraction for accessing the CPM registers. */

class Cpm_chip : public Cpm_chip_base
{
protected:
  Clock_base **_clocks;
  Cpm_regs _cpm_regs;

public:
  explicit Cpm_chip(l4_addr_t addr, Clock_base *clocks[]);

  const char *clock_type(enum Clock_identifiers clock);

  // Clock control.

  int have_clock(enum Clock_identifiers clock);
  void start_clock(enum Clock_identifiers clock);
  void stop_clock(enum Clock_identifiers clock);

  // Clock dividers.

  int get_parameters(enum Clock_identifiers clock, uint32_t parameters[]);
  int set_parameters(enum Clock_identifiers clock, int num_parameters,
                     uint32_t parameters[]);

  // Clock sources.

  uint8_t get_source(enum Clock_identifiers clock);
  void set_source(enum Clock_identifiers clock, uint8_t source);
  enum Clock_identifiers get_source_clock(enum Clock_identifiers clock);
  void set_source_clock(enum Clock_identifiers clock, enum Clock_identifiers source);

  // Source frequencies.

  uint64_t get_source_frequency(enum Clock_identifiers clock);

  // Output clock frequencies.

  uint64_t get_frequency(enum Clock_identifiers clock);
  int set_frequency(enum Clock_identifiers clock, uint64_t frequency);
};

#endif /* __cplusplus */
